@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPeerList PeerList List Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: peerlistmanagement-template.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: peerlistmanagement-template.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef INTERNAL_POOLTEMPLATE
@@ -57,12 +62,49 @@ void ST_CLASS(peerListManagementPrint)(
         struct ST_CLASS(PeerListManagement)* peerListManagement,
         FILE*                                fd,
         const unsigned int                   fields);
+void ST_CLASS(peerListManagementGetDescription)(
+        struct ST_CLASS(PeerListManagement)* peerListManagement,
+        char*                                buffer,
+        const size_t                         bufferSize);
+
 void ST_CLASS(peerListManagementVerify)(
         struct ST_CLASS(PeerListManagement)* peerListManagement);
 void ST_CLASS(peerListManagementClear)(
         struct ST_CLASS(PeerListManagement)* peerListManagement);
 size_t ST_CLASS(peerListManagementGetPeers)(
           const struct ST_CLASS(PeerListManagement)* peerListManagement);
+
+void ST_CLASS(peerListManagementActivateTimer)(
+        struct ST_CLASS(PeerListManagement)* peerListManagement,
+        struct ST_CLASS(PeerListNode)*       peerListNode,
+        const unsigned int                   timerCode,
+        const unsigned long long             timerTimeStamp);
+void ST_CLASS(peerListManagementDeactivateTimer)(
+        struct ST_CLASS(PeerListManagement)* peerListManagement,
+        struct ST_CLASS(PeerListNode)*       peerListNode);
+
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetFirstPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetLastPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetNextPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetPrevPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode);
+
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetFirstPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetLastPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetNextPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetPrevPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode);
+
 unsigned int ST_CLASS(peerListManagementRegisterPeerListNode)(
                 struct ST_CLASS(PeerListManagement)* peerListManagement,
                 const RegistrarIdentifierType        registrarIdentifier,
@@ -82,6 +124,8 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementFindNearestNextPeerLis
                                   struct ST_CLASS(PeerListManagement)* peerListManagement,
                                   const RegistrarIdentifierType        registrarIdentifier,
                                   const struct TransportAddressBlock*  transportAddressBlock);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetRandomPeerListNode)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement);
 unsigned int ST_CLASS(peerListManagementDeregisterPeerListNodeByPtr)(
                 struct ST_CLASS(PeerListManagement)* peerListManagement,
                 struct ST_CLASS(PeerListNode)*       peerListNode);
