@@ -1,28 +1,32 @@
-/*
- * An Efficient RSerPool Pool List Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: peerlistmanagement-template_impl.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: peerlistmanagement-template_impl.h 1289 2007-01-12 11:56:11Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-
 /*
 #define PRINT_SELECTION_RESULT
 */
@@ -73,6 +77,10 @@ static void ST_CLASS(peerListManagementPeerListNodeDisposer)(void* arg1,
                                                        peerListManagement->DisposerUserData);
       peerListNode->UserData = NULL;
    }
+   if(peerListNode->TakeoverProcess) {
+      free(peerListNode->TakeoverProcess);
+      peerListNode->TakeoverProcess = NULL;
+   }
    transportAddressBlockDelete(peerListNode->AddressBlock);
    free(peerListNode->AddressBlock);
    peerListNode->AddressBlock = NULL;
@@ -120,6 +128,39 @@ void ST_CLASS(peerListManagementPrint)(
 }
 
 
+/* ###### Insert PeerListNode into timer storage ######################### */
+void ST_CLASS(peerListManagementActivateTimer)(
+        struct ST_CLASS(PeerListManagement)* peerListManagement,
+        struct ST_CLASS(PeerListNode)*       peerListNode,
+        const unsigned int                   timerCode,
+        const unsigned long long             timerTimeStamp)
+{
+   return(ST_CLASS(peerListActivateTimer)(
+             &peerListManagement->List, peerListNode,
+             timerCode, timerTimeStamp));
+}
+
+
+/* ###### Remove PeerListNode from timer storage ######################### */
+void ST_CLASS(peerListManagementDeactivateTimer)(
+        struct ST_CLASS(PeerListManagement)* peerListManagement,
+        struct ST_CLASS(PeerListNode)*       peerListNode)
+{
+   return(ST_CLASS(peerListDeactivateTimer)(
+             &peerListManagement->List, peerListNode));
+}
+
+
+/* ###### Get textual description ######################################## */
+void ST_CLASS(peerListManagementGetDescription)(
+        struct ST_CLASS(PeerListManagement)* peerListManagement,
+        char*                                buffer,
+        const size_t                         bufferSize)
+{
+   return(ST_CLASS(peerListGetDescription)(&peerListManagement->List, buffer, bufferSize));
+}
+
+
 /* ###### Verify structures ############################################## */
 void ST_CLASS(peerListManagementVerify)(
         struct ST_CLASS(PeerListManagement)* peerListManagement)
@@ -136,6 +177,74 @@ size_t ST_CLASS(peerListManagementGetPeers)(
 }
 
 
+/* ###### Get first PeerListNode from Index ############################## */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetFirstPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement)
+{
+   return(ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(&peerListManagement->List));
+}
+
+
+/* ###### Get last PeerListNode from Index ############################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetLastPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement)
+{
+   return(ST_CLASS(peerListGetLastPeerListNodeFromIndexStorage)(&peerListManagement->List));
+}
+
+
+/* ###### Get next PeerListNode from Index ############################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetNextPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode)
+{
+   return(ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(&peerListManagement->List, peerListNode));
+}
+
+
+/* ###### Get previous PeerListNode from Index ########################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetPrevPeerListNodeFromIndexStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode)
+{
+   return(ST_CLASS(peerListGetPrevPeerListNodeFromIndexStorage)(&peerListManagement->List, peerListNode));
+}
+
+
+/* ###### Get first PeerListNode from Timer ############################## */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetFirstPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement)
+{
+   return(ST_CLASS(peerListGetFirstPeerListNodeFromTimerStorage)(&peerListManagement->List));
+}
+
+
+/* ###### Get last PeerListNode from Timer ############################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetLastPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement)
+{
+   return(ST_CLASS(peerListGetLastPeerListNodeFromTimerStorage)(&peerListManagement->List));
+}
+
+
+/* ###### Get next PeerListNode from Timer ############################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetNextPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode)
+{
+   return(ST_CLASS(peerListGetNextPeerListNodeFromTimerStorage)(&peerListManagement->List, peerListNode));
+}
+
+
+/* ###### Get previous PeerListNode from Timer ########################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetPrevPeerListNodeFromTimerStorage)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  struct ST_CLASS(PeerListNode)*       peerListNode)
+{
+   return(ST_CLASS(peerListGetPrevPeerListNodeFromTimerStorage)(&peerListManagement->List, peerListNode));
+}
+
+
 /* ###### Find PeerListNode ############################################## */
 struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementFindPeerListNode)(
                                   struct ST_CLASS(PeerListManagement)* peerListManagement,
@@ -175,6 +284,14 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementFindNearestNextPeerLis
 }
 
 
+/* ###### Get random PeerListNode ######################################## */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetRandomPeerListNode)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement)
+{
+   return(ST_CLASS(peerListGetRandomPeerListNode)(&peerListManagement->List));
+}
+
+
 /* ###### Get next timer time stamp ###################################### */
 unsigned long long ST_CLASS(peerListManagementGetNextTimerTimeStamp)(
                       struct ST_CLASS(PeerListManagement)* peerListManagement)
@@ -201,6 +318,7 @@ unsigned int ST_CLASS(peerListManagementRegisterPeerListNode)(
    struct ST_CLASS(PeerListNode) updatedPeerListNode;
    struct TransportAddressBlock* userTransport;
    unsigned int                  errorCode;
+   bool                          newPeer;
 
    /* ====== Handle dynamic entry ======================================== */
    if(flags & PLNF_DYNAMIC) {
@@ -216,12 +334,17 @@ unsigned int ST_CLASS(peerListManagementRegisterPeerListNode)(
                                                                            transportAddressBlock)) != NULL)
           ) && (!((*peerListNode)->Flags & PLNF_DYNAMIC)) ) {
          /* ====== Update of static entry ================================ */
+         newPeer = ((*peerListNode)->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER);
          ST_CLASS(peerListNodeNew)(&updatedPeerListNode,
                                     registrarIdentifier,
                                     (*peerListNode)->Flags, /* PLNF_DYNAMIC is never set here! */
                                     (*peerListNode)->AddressBlock);
          ST_CLASS(peerListUpdatePeerListNode)(&peerListManagement->List, *peerListNode,
                                               &updatedPeerListNode, &errorCode);
+         if(newPeer) {
+            /* New peer, because ID 0 changed to actual ID! */
+            (*peerListNode)->Flags |= PLNF_NEW;
+         }
          if(peerListManagement->Handlespace) {
             (*peerListNode)->OwnershipChecksum =
                ST_CLASS(poolHandlespaceNodeComputeOwnershipChecksum)(
@@ -298,6 +421,7 @@ unsigned int ST_CLASS(peerListManagementDeregisterPeerListNodeByPtr)(
                 struct ST_CLASS(PeerListNode)*       peerListNode)
 {
    unsigned int errorCode;
+   void*        userDataBackup;
 
    if(STN_METHOD(IsLinked)(&peerListNode->PeerListTimerStorageNode)) {
       ST_CLASS(peerListDeactivateTimer)(&peerListManagement->List,
@@ -310,10 +434,12 @@ unsigned int ST_CLASS(peerListManagementDeregisterPeerListNodeByPtr)(
       ST_CLASS(peerListRemovePeerListNode)(&peerListManagement->List, peerListNode);
       ST_CLASS(peerListNodeDelete)(peerListNode);
       /* Note: AddressBlock and Flags remain valid here! */
+      userDataBackup = peerListNode->UserData;   /* Preserve user data! */
       ST_CLASS(peerListNodeNew)(peerListNode,
                                 0,
                                 peerListNode->Flags,
                                 peerListNode->AddressBlock);
+      peerListNode->UserData = userDataBackup;
       ST_CLASS(peerListAddPeerListNode)(&peerListManagement->List, peerListNode, &errorCode);
       CHECK(errorCode == RSPERR_OKAY);
    }
