@@ -1,28 +1,32 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: poolhandlespacemanagement-template_impl.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: poolhandlespacemanagement-template_impl.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-
 /*
 #define PRINT_SELECTION_RESULT
 */
@@ -273,7 +277,17 @@ unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
 }
 
 
-/* ###### Print handlespace content ######################################## */
+/* ###### Get textual description ######################################## */
+void ST_CLASS(poolHandlespaceManagementGetDescription)(
+        const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+        char*                                             buffer,
+        const size_t                                      bufferSize)
+{
+   ST_CLASS(poolHandlespaceNodeGetDescription)(&poolHandlespaceManagement->Handlespace, buffer, bufferSize);
+}
+
+
+/* ###### Print handlespace content ###################################### */
 void ST_CLASS(poolHandlespaceManagementPrint)(
               const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
               FILE*                                             fd,
@@ -291,91 +305,6 @@ void ST_CLASS(poolHandlespaceManagementVerify)(
 }
 
 
-/* ###### Get number of pools ############################################ */
-size_t ST_CLASS(poolHandlespaceManagementGetPools)(
-          const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
-{
-   return(ST_CLASS(poolHandlespaceNodeGetPoolNodes)(&poolHandlespaceManagement->Handlespace));
-}
-
-
-/* ###### Get number of pool elements #################################### */
-size_t ST_CLASS(poolHandlespaceManagementGetPoolElements)(
-          const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
-{
-   return(ST_CLASS(poolHandlespaceNodeGetPoolElementNodes)(&poolHandlespaceManagement->Handlespace));
-}
-
-
-/* ###### Get number of owned pool elements ############################## */
-size_t ST_CLASS(poolHandlespaceManagementGetOwnedPoolElements)(
-          const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
-{
-   return(ST_CLASS(poolHandlespaceNodeGetOwnedPoolElementNodes)(&poolHandlespaceManagement->Handlespace));
-}
-
-
-/* ###### Get number of pool elements of given pool ###################### */
-size_t ST_CLASS(poolHandlespaceManagementGetPoolElementsOfPool)(
-          struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
-          const struct PoolHandle*                    poolHandle)
-{
-   return(ST_CLASS(poolHandlespaceNodeGetPoolElementNodesOfPool)(
-             &poolHandlespaceManagement->Handlespace,
-             poolHandle));
-}
-
-
-/* ###### Get number of pool elements of given connection ################ */
-size_t ST_CLASS(poolHandlespaceManagementGetPoolElementsOfConnection)(
-          struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
-          const int                                   socketDescriptor,
-          const sctp_assoc_t                          assocID)
-{
-   return(ST_CLASS(poolHandlespaceNodeGetConnectionNodesForConnection)(
-             &poolHandlespaceManagement->Handlespace,
-             socketDescriptor, assocID));
-}
-
-
-/* ###### Registration ################################################### */
-unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElementByPtr)(
-                struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
-                const struct PoolHandle*                    poolHandle,
-                const struct ST_CLASS(PoolElementNode)*     originalPoolElementNode,
-                const unsigned long long                    currentTimeStamp,
-                struct ST_CLASS(PoolElementNode)**          poolElementNode)
-{
-   return(ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
-             poolHandlespaceManagement,
-             poolHandle,
-             originalPoolElementNode->HomeRegistrarIdentifier,
-             originalPoolElementNode->Identifier,
-             originalPoolElementNode->RegistrationLife,
-             &originalPoolElementNode->PolicySettings,
-             originalPoolElementNode->UserTransport,
-             originalPoolElementNode->RegistratorTransport,
-             originalPoolElementNode->ConnectionSocketDescriptor,
-             originalPoolElementNode->ConnectionAssocID,
-             currentTimeStamp,
-             poolElementNode));
-}
-
-
-/* ###### Get next timer time stamp ###################################### */
-unsigned long long ST_CLASS(poolHandlespaceManagementGetNextTimerTimeStamp)(
-                      struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
-{
-   const struct ST_CLASS(PoolElementNode)* nextTimer =
-      ST_CLASS(poolHandlespaceNodeGetFirstPoolElementTimerNode)(
-         &poolHandlespaceManagement->Handlespace);
-   if(nextTimer != NULL) {
-      return(nextTimer->TimerTimeStamp);
-   }
-   return(~0);
-}
-
-
 /* ###### Find pool element ############################################## */
 struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementFindPoolElement)(
                                      struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
@@ -414,6 +343,30 @@ unsigned int ST_CLASS(poolHandlespaceManagementDeregisterPoolElementByPtr)(
 }
 
 
+/* ###### Registration ################################################### */
+unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElementByPtr)(
+                struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                const struct PoolHandle*                    poolHandle,
+                const struct ST_CLASS(PoolElementNode)*     originalPoolElementNode,
+                const unsigned long long                    currentTimeStamp,
+                struct ST_CLASS(PoolElementNode)**          poolElementNode)
+{
+   return(ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
+             poolHandlespaceManagement,
+             poolHandle,
+             originalPoolElementNode->HomeRegistrarIdentifier,
+             originalPoolElementNode->Identifier,
+             originalPoolElementNode->RegistrationLife,
+             &originalPoolElementNode->PolicySettings,
+             originalPoolElementNode->UserTransport,
+             originalPoolElementNode->RegistratorTransport,
+             originalPoolElementNode->ConnectionSocketDescriptor,
+             originalPoolElementNode->ConnectionAssocID,
+             currentTimeStamp,
+             poolElementNode));
+}
+
+
 /* ###### Deregistration ################################################# */
 unsigned int ST_CLASS(poolHandlespaceManagementDeregisterPoolElement)(
                 struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
@@ -433,6 +386,34 @@ unsigned int ST_CLASS(poolHandlespaceManagementDeregisterPoolElement)(
 }
 
 
+/* ###### Update PoolElementNode's ownership ############################# */
+void ST_CLASS(poolHandlespaceManagementUpdateOwnershipOfPoolElementNode)(
+              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+              struct ST_CLASS(PoolElementNode)*           poolElementNode,
+              const RegistrarIdentifierType               newHomeRegistrarIdentifier)
+{
+   return(ST_CLASS(poolHandlespaceNodeUpdateOwnershipOfPoolElementNode)(
+             &poolHandlespaceManagement->Handlespace,
+             poolElementNode,
+             newHomeRegistrarIdentifier));
+}
+
+
+/* ###### Update PoolElementNode's connection ############################ */
+void ST_CLASS(poolHandlespaceManagementUpdateConnectionOfPoolElementNode)(
+        struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+        struct ST_CLASS(PoolElementNode)*           poolElementNode,
+        const int                                   connectionSocketDescriptor,
+        const sctp_assoc_t                          connectionAssocID)
+{
+   return(ST_CLASS(poolHandlespaceNodeUpdateConnectionOfPoolElementNode)(
+             &poolHandlespaceManagement->Handlespace,
+             poolElementNode,
+             connectionSocketDescriptor,
+             connectionAssocID));
+}
+
+
 /* ###### Handle Resolution ############################################## */
 unsigned int ST_CLASS(poolHandlespaceManagementHandleResolution)(
                 struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
@@ -733,3 +714,166 @@ size_t ST_CLASS(poolHandlespaceManagementPurgeMarkedPoolElementNodes)(
    }
    return(count);
 }
+
+
+/* ###### Get number of pools ############################################ */
+size_t ST_CLASS(poolHandlespaceManagementGetPools)(
+          const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetPoolNodes)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get number of pool elements #################################### */
+size_t ST_CLASS(poolHandlespaceManagementGetPoolElements)(
+          const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetPoolElementNodes)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get number of owned pool elements ############################## */
+size_t ST_CLASS(poolHandlespaceManagementGetOwnedPoolElements)(
+          const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetOwnedPoolElementNodes)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get number of pool elements of given pool ###################### */
+size_t ST_CLASS(poolHandlespaceManagementGetPoolElementsOfPool)(
+          struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+          const struct PoolHandle*                    poolHandle)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetPoolElementNodesOfPool)(
+             &poolHandlespaceManagement->Handlespace,
+             poolHandle));
+}
+
+
+/* ###### Get number of pool elements of given connection ################ */
+size_t ST_CLASS(poolHandlespaceManagementGetPoolElementsOfConnection)(
+          struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+          const int                                   socketDescriptor,
+          const sctp_assoc_t                          assocID)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetConnectionNodesForConnection)(
+             &poolHandlespaceManagement->Handlespace,
+             socketDescriptor, assocID));
+}
+
+
+/* ###### Get first PoolNode ############################################# */
+struct ST_CLASS(PoolNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolNode)(
+                              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetFirstPoolNode)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get next PoolNode ############################################## */
+struct ST_CLASS(PoolNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolNode)(
+                              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                              struct ST_CLASS(PoolNode)*                  poolNode)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetNextPoolNode)(&poolHandlespaceManagement->Handlespace,
+                                                       poolNode));
+}
+
+
+/* ###### Get first timer ################################################ */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementTimerNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetFirstPoolElementTimerNode)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get previous timer ############################################# */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementTimerNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetNextPoolElementTimerNode)(&poolHandlespaceManagement->Handlespace,
+                                                                   poolElementNode));
+}
+
+
+/* ###### Get next timer time stamp ###################################### */
+unsigned long long ST_CLASS(poolHandlespaceManagementGetNextTimerTimeStamp)(
+                      struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   const struct ST_CLASS(PoolElementNode)* nextTimer =
+      ST_CLASS(poolHandlespaceNodeGetFirstPoolElementTimerNode)(
+         &poolHandlespaceManagement->Handlespace);
+   if(nextTimer != NULL) {
+      return(nextTimer->TimerTimeStamp);
+   }
+   return(~0);
+}
+
+
+/* ###### Get first connection ############################################ */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementConnectionNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetFirstPoolElementConnectionNode)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get previous connection ######################################## */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementConnectionNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetNextPoolElementConnectionNode)(&poolHandlespaceManagement->Handlespace,
+                                                                        poolElementNode));
+}
+
+
+/* ###### Get first ownership node of given home PR identifier ########### */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementOwnershipNodeForIdentifier)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     const RegistrarIdentifierType               homeRegistrarIdentifier)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetFirstPoolElementOwnershipNodeForIdentifier)(&poolHandlespaceManagement->Handlespace,
+                                                                                     homeRegistrarIdentifier));
+}
+
+
+/* ###### Get next connection of same home PR identifier ################# */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementConnectionNodeForSameConnection)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetNextPoolElementConnectionNodeForSameConnection)(&poolHandlespaceManagement->Handlespace,
+                                                                                         poolElementNode));
+}
+
+
+/* ###### Get first ownership ############################################ */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementOwnershipNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetFirstPoolElementOwnershipNode)(&poolHandlespaceManagement->Handlespace));
+}
+
+
+/* ###### Get previous ownership ######################################### */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementOwnershipNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetNextPoolElementOwnershipNode)(&poolHandlespaceManagement->Handlespace,
+                                                                       poolElementNode));
+}
+
+
+/* ###### Get next ownership of same home PR identifier ################## */
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementOwnershipNodeForSameIdentifier)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode)
+{
+   return(ST_CLASS(poolHandlespaceNodeGetNextPoolElementOwnershipNodeForSameIdentifier)(&poolHandlespaceManagement->Handlespace,
+                                                                                        poolElementNode));
+}
