@@ -1,4 +1,4 @@
-/* $Id: registrar-asap.c 1744 2008-02-15 08:51:50Z dreibh $
+/* $Id: registrar-asap.c 1863 2008-08-26 13:24:38Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -28,6 +28,7 @@
  */
 
 #include "registrar.h"
+#include "poolhandle.h"
 
 
 /* ###### ASAP Announce timer ############################################ */
@@ -49,6 +50,8 @@ void registrarHandleASAPAnnounceTimer(struct Dispatcher* dispatcher,
       message->RegistrarIdentifier = registrar->ServerID;
       messageLength = rserpoolMessage2Packet(message);
       if(messageLength > 0) {
+         registrarWriteActionLog(registrar, "Send", "MulticastASAP", "Announce", "", 0, 0, 0,
+                                 NULL, 0, message->RegistrarIdentifier, 0, 0, 0);
          if(sendmulticast(
                registrar->ASAPAnnounceSocket,
                registrar->ASAPAnnounceSocketFamily,
@@ -56,7 +59,8 @@ void registrarHandleASAPAnnounceTimer(struct Dispatcher* dispatcher,
                messageLength,
                0,
                (struct sockaddr*)&registrar->ASAPAnnounceAddress,
-               getSocklen((struct sockaddr*)&registrar->ASAPAnnounceAddress)) <= 0) {
+               getSocklen((struct sockaddr*)&registrar->ASAPAnnounceAddress),
+               registrar->AnnounceTTL) <= 0) {
             LOG_WARNING
             logerror("Unable to send announce");
             LOG_END
@@ -91,24 +95,29 @@ void registrarHandlePoolElementEvent(struct Dispatcher* dispatcher,
             &registrar->Handlespace.Handlespace,
             poolElementNode);
 
-         registrarSendASAPEndpointKeepAlive(registrar, poolElementNode, false);
+         registrarWriteActionLog(registrar, "Send", "ASAP", "EndpointKeepAlive", "KeepAliveTransmissionTimer", 0, 0, registrar->EndpointKeepAliveTimeoutInterval,
+                                 &poolElementNode->OwnerPoolNode->Handle,poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
 
+         registrarSendASAPEndpointKeepAlive(registrar, poolElementNode, false);
+         poolElementNode->LastKeepAliveTransmission = getMicroTime();
          ST_CLASS(poolHandlespaceNodeActivateTimer)(
             &registrar->Handlespace.Handlespace,
             poolElementNode,
             PENT_KEEPALIVE_TIMEOUT,
-            getMicroTime() + registrar->EndpointKeepAliveTimeoutInterval);
+            poolElementNode->LastKeepAliveTransmission + registrar->EndpointKeepAliveTimeoutInterval);
       }
 
       else if( (poolElementNode->TimerCode == PENT_KEEPALIVE_TIMEOUT) ||
                (poolElementNode->TimerCode == PENT_EXPIRY) ) {
          if(poolElementNode->TimerCode == PENT_KEEPALIVE_TIMEOUT) {
             LOG_ACTION
-            fprintf(stdlog, "Keep alive timeout expired for pool element $%08x of pool ",
-                  poolElementNode->Identifier);
+            fprintf(stdlog, "Keep-alive timeout expired for pool element $%08x of pool ",
+                    poolElementNode->Identifier);
             poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdlog);
             fputs(" -> removing it\n", stdlog);
             LOG_END
+            registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "KeepAliveTimeoutTimer", 0, 0, registrar->EndpointKeepAliveTimeoutInterval,
+                                    &poolElementNode->OwnerPoolNode->Handle, poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
          }
          else {
             LOG_ACTION
@@ -117,6 +126,8 @@ void registrarHandlePoolElementEvent(struct Dispatcher* dispatcher,
             poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdlog);
             fputs(" -> removing it\n", stdlog);
             LOG_END
+            registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "ExpiryTimer", 0, 0, registrar->TakeoverExpiryInterval,
+                                    &poolElementNode->OwnerPoolNode->Handle, poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
          }
 
          if(poolElementNode->HomeRegistrarIdentifier == registrar->ServerID) {
@@ -198,6 +209,9 @@ void registrarRemovePoolElementsOfConnection(struct Registrar*  registrar,
             registrarSendENRPHandleUpdate(registrar, poolElementNode, PNUP_DEL_PE);
          }
 
+         registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "Disconnect", 0, 0, 0,
+                                 &poolElementNode->OwnerPoolNode->Handle, poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
+
          result = ST_CLASS(poolHandlespaceManagementDeregisterPoolElementByPtr)(
                      &registrar->Handlespace,
                      poolElementNode);
@@ -251,6 +265,9 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
    fprintf(stdlog, "/$%08x\n", message->PoolElementPtr->Identifier);
    LOG_END
 
+   registrarWriteActionLog(registrar, "Recv", "ASAP", "Registration", "Requested", 0, 0, 0,
+                           &message->Handle, message->PoolElementPtr->Identifier, registrar->ServerID, 0, 0, 0);
+
    message->Type       = AHT_REGISTRATION_RESPONSE;
    message->Flags      = 0x00;
    message->Error      = RSPERR_OKAY;
@@ -318,6 +335,7 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
                fputs("\n", stdlog);
                LOG_END
 
+
                /* ====== Tune SCTP association ============================== */
 /*
                tags[0].Tag = TAG_TuneSCTP_MinRTO;
@@ -416,6 +434,8 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
       if(message->Error) {
          message->Flags |= AHF_REGISTRATION_REJECT;
       }
+      registrarWriteActionLog(registrar, "Recv", "ASAP", "RegistrationResponse", "Requested", 0, 0, 0,
+                              &message->Handle, message->PoolElementPtr->Identifier, registrar->ServerID, 0, 0, message->Error);
       if(rserpoolMessageSend(IPPROTO_SCTP, fd, assocID, 0, 0, 0, message) == false) {
          LOG_WARNING
          logerror("Sending RegistrationResponse failed");
@@ -447,6 +467,9 @@ void registrarHandleASAPDeregistration(struct Registrar*       registrar,
    fprintf(stdlog, "/$%08x\n", message->Identifier);
    LOG_END
 
+   registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "Requested", 0, 0, 0,
+                           &message->Handle, message->Identifier, registrar->ServerID, 0, 0, 0);
+
    message->Type  = AHT_DEREGISTRATION_RESPONSE;
    message->Flags = 0x00;
 
@@ -531,6 +554,8 @@ void registrarHandleASAPDeregistration(struct Registrar*       registrar,
       LOG_END
    }
 
+   registrarWriteActionLog(registrar, "Send", "ASAP", "DeregistrationResponse", "Requested", message->Flags, 0, 0,
+                           &message->Handle, message->Identifier, registrar->ServerID, 0, 0, message->Error);
    if(rserpoolMessageSend(IPPROTO_SCTP, fd, assocID, 0, 0, 0, message) == false) {
       LOG_WARNING
       logerror("Sending DeregistrationResponse failed");
@@ -563,6 +588,9 @@ void registrarHandleASAPHandleResolution(struct Registrar*       registrar,
            (unsigned int)items, (unsigned int)message->Addresses);
    LOG_END
 
+   registrarWriteActionLog(registrar, "Recv", "ASAP", "HandleResolution", "Requested", 0, items, 0,
+                           &message->Handle, 0, 0, 0, 0, 0);
+
 
    registrar->HandleResolutionCount++;
 
@@ -611,6 +639,10 @@ void registrarHandleASAPHandleResolution(struct Registrar*       registrar,
       LOG_END
    }
 
+   registrarWriteActionLog(registrar, "Send", "ASAP", "HandleResolutionResponse", "Requested", 0, poolElementNodes, 0,
+                           &message->Handle, (poolElementNodes > 0) ? poolElementNodeArray[0]->Identifier : 0,
+                           0, 0, 0, message->Error);
+
    if(rserpoolMessageSend(IPPROTO_SCTP, fd, assocID, 0, 0, 0, message) == false) {
       LOG_WARNING
       logerror("Sending handle resolution response failed");
@@ -626,6 +658,7 @@ void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
                                              struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNode;
+   const unsigned long long          now = getMicroTime();
 
    LOG_VERBOSE2
    fprintf(stdlog, "Got EndpointKeepAliveAck for pool element $%08x of pool ",
@@ -643,6 +676,10 @@ void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
       fputs("EndpointKeepAlive successful, resetting timer\n", stdlog);
       LOG_END
 
+      registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointKeepAliveAck", "InTime", 0, 0,
+                              now - poolElementNode->LastKeepAliveTransmission,
+                              &message->Handle, message->Identifier, 0, 0, 0, 0);
+
       ST_CLASS(poolHandlespaceNodeDeactivateTimer)(
          &registrar->Handlespace.Handlespace,
          poolElementNode);
@@ -656,6 +693,9 @@ void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
                       &registrar->Handlespace));
    }
    else {
+      registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointKeepAliveAck", "TooLateAlreadyRemoved", 0, 0, 0,
+                              &message->Handle, message->Identifier, 0, 0, 0, 0);
+
       LOG_WARNING
       fprintf(stdlog,
               "EndpointKeepAliveAck for not-existing pool element $%08x of pool ",
@@ -745,6 +785,9 @@ void registrarHandleASAPEndpointUnreachable(struct Registrar*       registrar,
 
    registrar->FailureReportCount++;
 
+   registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointUnreachable", "", 0, registrar->FailureReportCount, 0,
+                           &message->Handle, message->Identifier, 0, 0, 0, 0);
+
    poolElementNode = ST_CLASS(poolHandlespaceManagementFindPoolElement)(
                         &registrar->Handlespace,
                         &message->Handle,
