@@ -1,4 +1,4 @@
-/* $Id: registrar.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: registrar.h 1816 2008-06-24 16:27:21Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -49,6 +49,7 @@
 #include <ext_socket.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
+#include <bzlib.h>
 
 
 /* Exit immediately on Ctrl-C. No clean shutdown. */
@@ -72,6 +73,7 @@
 #define REGISTRAR_DEFAULT_MENTOR_DISCOVERY_TIMEOUT                    5000000
 #define REGISTRAR_DEFAULT_TAKEOVER_EXPIRY_INTERVAL                    5000000
 #define REGISTRAR_DEFAULT_AUTOCLOSE_TIMEOUT                         300000000
+#define REGISTRAR_DEFAULT_ANNOUNCE_TTL                                     30
 
 
 struct Registrar
@@ -109,6 +111,7 @@ struct Registrar
    bool                                       InStartupPhase;
    RegistrarIdentifierType                    MentorServerID;
 
+   int                                        AnnounceTTL;
    size_t                                     DistanceStep;
    size_t                                     MaxBadPEReports;
    unsigned long long                         AutoCloseTimeout;
@@ -126,7 +129,13 @@ struct Registrar
    unsigned long long                         MentorDiscoveryTimeout;
    unsigned long long                         TakeoverExpiryInterval;
 
+   FILE*                                      ActionLogFile;
+   BZFILE*                                    ActionLogBZFile;
+   unsigned long long                         ActionLogLine;
+   unsigned long long                         ActionLogLastActivity;
+   unsigned long long                         ActionLogStartTime;
    FILE*                                      StatsFile;
+   BZFILE*                                    StatsBZFile;
    struct Timer                               StatsTimer;
    unsigned long long                         StatsLine;
    unsigned long long                         StatsStartTime;
@@ -158,7 +167,10 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
                                const union sockaddr_union*    asapAnnounceAddress,
                                const bool                     enrpAnnounceViaMulticast,
                                const union sockaddr_union*    enrpMulticastAddress,
+                               FILE*                          actionLogFile,
+                               BZFILE*                        actionLogBZFile,
                                FILE*                          statsFile,
+                               BZFILE*                        statsBZFile,
                                unsigned int                   statsInterval
 #ifdef ENABLE_CSP
                                , const unsigned int           cspReportInterval,
@@ -183,6 +195,24 @@ void registrarUpdateDistance(struct Registrar*                       registrar,
                              unsigned int*                           distance);
 
 
+/* ###### Action Log ##################################################### */
+void registrarBeginActionLog(struct Registrar* registrar);
+void registrarWriteActionLog(struct Registrar*         registrar,
+                             const char*               direction,
+                             const char*               protocol,
+                             const char*               action,
+                             const char*               reason,
+                             const uint32_t            flags,
+                             const unsigned long long  counter,
+                             const unsigned long long  timeValue,
+                             const struct PoolHandle*  poolHandle,
+                             PoolElementIdentifierType poolElementID,
+                             RegistrarIdentifierType   senderID,
+                             RegistrarIdentifierType   receiverID,
+                             RegistrarIdentifierType   targetID,
+                             unsigned int              errorCode);
+
+
 /* ###### Core ########################################################### */
 void registrarDumpHandlespace(struct Registrar* registrar);
 void registrarDumpPeers(struct Registrar* registrar);
