@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: rserpool.h 1732 2008-02-14 12:14:23Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: rserpool.h 1435 2007-06-05 13:20:58Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef RSERPOOL_H
@@ -41,8 +45,21 @@ extern "C" {
 #endif
 
 
-#define RSPLIB_VERSION  1
-#define RSPLIB_REVISION 9
+#define RSPLIB_VERSION  2
+#define RSPLIB_REVISION 3
+
+
+/*
+   ##########################################################################
+   #### RSerPool DEFINITIONS                                             ####
+   ##########################################################################
+*/
+
+#define ASAP_PORT 3863
+#define ENRP_PORT 9901
+
+#define ASAP_ANNOUNCE_MULTICAST_ADDRESS "239.0.0.50:3863"
+#define ENRP_ANNOUNCE_MULTICAST_ADDRESS "239.0.0.51:9901"
 
 
 /*
@@ -176,6 +193,7 @@ unsigned int rsp_pe_failure(const unsigned char* poolHandle,
   * @param poolHandleSize Pool handle size.
   * @param rserpoolAddrInfo Pointer to variable to store pointer to first rsp_addrinfo to.
   * @param items Desired number of PE entries to obtain.
+  * @param staleCacheValue Stale cache value in milliseconds.
   * @return Number of PE entries obtained in case of success; -1 in case of an error.
   *
   * @see rsp_freeaddrinfo
@@ -183,7 +201,8 @@ unsigned int rsp_pe_failure(const unsigned char* poolHandle,
 int rsp_getaddrinfo(const unsigned char*  poolHandle,
                     const size_t          poolHandleSize,
                     struct rsp_addrinfo** rserpoolAddrInfo,
-                    const size_t          items);
+                    const size_t          items,
+                    const unsigned int    staleCacheValue);
 
 /**
   * Free rsp_addrinfo structure.
@@ -456,11 +475,13 @@ int rsp_accept(int sd,
   * @param sd RSerPool socket descriptor.
   * @param poolHandle Pool handle.
   * @param poolHandleSize Size of the pool handle in bytes.
+  * @param staleCacheValue Stale cache value in milliseconds.
   * @return 0 in case of success; -1 in case of an error.
   */
 int rsp_connect(int                  sd,
                 const unsigned char* poolHandle,
-                const size_t         poolHandleSize);
+                const size_t         poolHandleSize,
+                const unsigned int   staleCacheValue);
 
 /**
   * Check, if a RSerPool socket has received a cookie from its PE.
@@ -470,13 +491,21 @@ int rsp_connect(int                  sd,
   */
 int rsp_has_cookie(int sd);
 
+
+#define FFF_NONE        0                 /* Normal failover, no report */
+#define FFF_UNREACHABLE (1 << 0)          /* Report unreachable PE      */
+
 /**
   * For a failover to a new Pool Element.
   *
   * @param sd RSerPool socket descriptor.
+  * @param flags Flags.
+  * @param staleCacheValue Stale cache value in milliseconds.
   * @return 0 in case of success; -1 in case of an error.
   */
-int rsp_forcefailover(int sd);
+int rsp_forcefailover(int                sd,
+                      const unsigned int flags,
+                      const unsigned int staleCacheValue);
 
 
 /* #######################################################################
@@ -494,6 +523,7 @@ int rsp_forcefailover(int sd);
   * @param sctpPPID SCTP payload protocol identifier.
   * @param sctpStreamID SCTP stream ID.
   * @param sctpTimeToLive SCTP time to live (ignored, if Pr-SCTP is not available).
+  * @param sctpFlags SCTP flags.
   * @param timeout Timeout in milliseconds; -1 for infinite.
   * @return length of the sent data in case of success; -1 in case of an error.
   */
@@ -505,6 +535,7 @@ ssize_t rsp_sendmsg(int                sd,
                     uint32_t           sctpPPID,
                     uint16_t           sctpStreamID,
                     uint32_t           sctpTimeToLive,
+                    uint16_t           sctpFlags,
                     int                timeout);
 
 /**
