@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: rsputilities.c 1733 2008-02-14 15:28:10Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: rsputilities.c 1314 2007-02-03 20:54:55Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "tdtypes.h"
@@ -106,6 +110,9 @@ static bool initComponentStatusReporter(struct rsp_info* info,
                   (const char*)&parameter[11]);
          return(false);
       }
+      if(getPort(&cspServerAddress.sa) == 0) {
+         setPort(&cspServerAddress.sa, 2960);
+      }
       info->ri_csp_server = &cspServerAddress.sa;
    }
    else if(!(strncmp(parameter, "-cspinterval=", 13))) {
@@ -133,7 +140,28 @@ static bool initComponentStatusReporter(struct rsp_info* info,
 /* ###### Initialize rsp_info ############################################ */
 void rsp_initinfo(struct rsp_info* info)
 {
+#ifdef ENABLE_CSP
+   const char*                 cspServer   = getenv("CSP_SERVER");
+   const char*                 cspInterval = getenv("CSP_INTERVAL");
+   static union sockaddr_union cspServerAddress;
+#endif
    memset(info, 0, sizeof(struct rsp_info));
+#ifdef ENABLE_CSP
+   if(cspServer) {
+      if(!string2address(cspServer, &cspServerAddress)) {
+         fprintf(stderr,
+                  "ERROR: Bad CSP report address %s! Use format <address:port>.\n",
+                  cspServer);
+      }
+      info->ri_csp_server = &cspServerAddress.sa;
+   }
+   if(cspInterval) {
+      info->ri_csp_interval = atol(cspInterval);
+      if(info->ri_csp_interval < 250) {
+         info->ri_csp_interval = 250;
+      }
+   }
+#endif
 }
 
 
@@ -190,6 +218,9 @@ int rsp_initarg(struct rsp_info* info, const char* arg)
             fprintf(stderr, "ERROR: Bad ASAP announce setting %s\n", arg);
             exit(1);
          }
+         if(getPort(&asapAnnounceAddress.sa) == 0) {
+            setPort(&asapAnnounceAddress.sa, ASAP_PORT);
+         }
          info->ri_registrar_announce = (struct sockaddr*)&asapAnnounceAddress;
          info->ri_disable_autoconfig = 0;
       }
