@@ -1,4 +1,4 @@
-/* $Id: server.cc 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: server.cc 1793 2008-06-08 16:50:59Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -39,6 +39,10 @@
 #include "componentstatuspackets.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
 #include <math.h>
 
 
